# Generated by Django 5.0.6 on 2024-06-03 08:52

import django.db.models.deletion
import django.utils.timezone
from django.conf import settings
from django.db import migrations, models


class Migration(migrations.Migration):
    replaces = [
        ("user_messages", "0001_initial"),
        ("user_messages", "0002_alter_message_index_together"),
        ("user_messages", "0003_alter_message_index_together_and_more"),
    ]

    initial = True

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
    ]

    operations = [
        migrations.CreateModel(
            name="Message",
            fields=[
                (
                    "id",
                    models.AutoField(
                        auto_created=True,
                        primary_key=True,
                        serialize=False,
                        verbose_name="ID",
                    ),
                ),
                (
                    "created_at",
                    models.DateTimeField(
                        default=django.utils.timezone.now, verbose_name="created at"
                    ),
                ),
                (
                    "delivered_at",
                    models.DateTimeField(
                        blank=True, null=True, verbose_name="delivered at"
                    ),
                ),
                ("level", models.IntegerField(verbose_name="level")),
                ("message", models.TextField(verbose_name="message")),
                ("extra_tags", models.TextField(blank=True, verbose_name="extra tags")),
                ("_metadata", models.TextField(blank=True, verbose_name="meta data")),
                (
                    "deliver_once",
                    models.BooleanField(default=True, verbose_name="deliver once"),
                ),
                (
                    "user",
                    models.ForeignKey(
                        on_delete=django.db.models.deletion.CASCADE,
                        related_name="+",
                        to=settings.AUTH_USER_MODEL,
                        verbose_name="user",
                    ),
                ),
            ],
            options={
                "verbose_name": "message",
                "verbose_name_plural": "messages",
                # 'index_together': set(),
                "indexes": [
                    models.Index(
                        models.F("user"),
                        models.F("delivered_at"),
                        name="user_delivered_at",
                    )
                ],
            },
        ),
    ]



import json

from django.contrib.auth import SESSION_KEY
from django.contrib.messages import api, constants
from django.db import models
from django.utils import timezone
from django.utils.functional import SimpleLazyObject


def add_message(user, level, message, extra_tags="", *, deliver_once=True, meta=None):
    from user_messages.models import Message

    Message.objects.create(
        level=level or 20,  # INFO
        message=message,
        extra_tags=extra_tags,
        _metadata=json.dumps(meta or {}),
        deliver_once=deliver_once,
        **{"user" if isinstance(user, models.Model) else "user_id": user},
    )


def get_messages(*, request=None, user=None):
    assert bool(request) != bool(user), "Pass exactly one of request or user"
    _nonlocal = (user,)

    def fetch():
        messages = []
        (user,) = _nonlocal

        if request is not None:
            messages.extend(api.get_messages(request))
            if request.session.get(SESSION_KEY) and request.user.is_authenticated:
                user = request.user

        if user is not None:
            from user_messages.models import Message

            user_messages = Message.objects.filter(
                user=user, delivered_at__isnull=True
            ).order_by("pk")
            messages.extend(user_messages)
            if any(m.deliver_once for m in user_messages):
                user_messages.filter(deliver_once=True).update(
                    delivered_at=timezone.now()
                )

        return messages

    return SimpleLazyObject(fetch)


def _create_shortcut(level):
    def helper(user, message, extra_tags="", *, deliver_once=True, meta=None):
        add_message(
            user,
            level,
            message,
            extra_tags=extra_tags,
            deliver_once=deliver_once,
            meta=meta,
        )

    return helper


debug = _create_shortcut(constants.DEBUG)
info = _create_shortcut(constants.INFO)
success = _create_shortcut(constants.SUCCESS)
warning = _create_shortcut(constants.WARNING)
error = _create_shortcut(constants.ERROR)



from django.apps import AppConfig
from django.core import checks
from django.template import engines
from django.template.backends.django import DjangoTemplates
from django.utils.text import capfirst
from django.utils.translation import gettext_lazy as _


@checks.register()
def check_context_processors(app_configs, **kwargs):
    errors = []

    for engine in engines.all():
        if isinstance(engine, DjangoTemplates):
            context_processors = engine.engine.context_processors
            try:
                dj = context_processors.index(
                    "django.contrib.messages.context_processors.messages"
                )
                um = context_processors.index(
                    "user_messages.context_processors.messages"
                )
            except ValueError:
                continue
            if um < dj:
                errors.append(
                    checks.Error(
                        "Insert 'user_messages.context_processors.messages' after 'django.contrib.messages.context_processors.messages' so that the 'messages' context variable actually contains user messages.",
                        id="user_messages.E002",
                    )
                )

    return errors


class UserMessagesConfig(AppConfig):
    default_auto_field = "django.db.models.AutoField"
    name = "user_messages"
    verbose_name = capfirst(_("user messages"))



from django.contrib import admin

from user_messages import models


@admin.register(models.Message)
class MessageAdmin(admin.ModelAdmin):
    list_display = (
        "created_at",
        "delivered_at",
        "user",
        "level",
        "message",
        "deliver_once",
    )
    ordering = ["-created_at"]
    raw_id_fields = ("user",)



import json

from django.conf import settings
from django.contrib.messages.storage.base import LEVEL_TAGS
from django.db import models
from django.utils import timezone
from django.utils.functional import cached_property
from django.utils.translation import gettext_lazy as _


class Message(models.Model):
    created_at = models.DateTimeField(_("created at"), default=timezone.now)
    delivered_at = models.DateTimeField(_("delivered at"), blank=True, null=True)
    user = models.ForeignKey(
        settings.AUTH_USER_MODEL,
        on_delete=models.CASCADE,
        verbose_name=_("user"),
        related_name="+",
    )
    level = models.IntegerField(_("level"))
    message = models.TextField(_("message"))
    extra_tags = models.TextField(_("extra tags"), blank=True)
    _metadata = models.TextField(_("meta data"), blank=True)
    deliver_once = models.BooleanField(_("deliver once"), default=True)

    class Meta:
        indexes = [
            models.Index("user", "delivered_at", name="user_delivered_at"),
        ]
        verbose_name = _("message")
        verbose_name_plural = _("messages")

    def __str__(self):
        # Duck typing django.contrib.messages.storage.base.Message
        return self.message

    @property
    def level_tag(self):
        return LEVEL_TAGS.get(self.level, "")

    @property
    def tags(self):
        return " ".join(tag for tag in [self.extra_tags, self.level_tag] if tag)

    @cached_property
    def meta(self):
        if self._metadata:
            try:
                return json.loads(self._metadata)
            except Exception:
                pass
        return {}
